package Data::Array2ArrayMap::Hash::MultiKey;

use 5.006;
use strict;
use warnings;

use Carp;
use Data::Array2ArrayMap;
use Clone;

use vars qw/$VERSION @ISA *delete/;
$VERSION = '0.12';
@ISA = qw/Data::Array2ArrayMap Clone/;

=head2

This is backend of Data::Array2ArrayMap, 
implemented on base of Hash::MultiKey code 
[Copyright (C) 2003, Xavier Noria <fxn@cpan.org>]
that stores data in single hash using pack() of the path.

See Data::Array2ArrayMap documentation for its interface.

=cut

sub has_find {return 0}
sub has_clone {return 1}
sub can_store_empty_array() {return 1}
sub can_list_paths() {return 1}

sub get_ref_if {
    my ($self, $keys) = @_;
    croak "Bad multi-key\n" unless ref $keys eq 'ARRAY';
    $self->{pack 'N' . ('w/a*' x @$keys), scalar(@$keys), @$keys};
}

sub get_ref {
    my ($self, $keys) = @_;
    croak "Bad multi-key\n" unless ref $keys eq 'ARRAY';
    my $retval=$self->{pack 'N' . ('w/a*' x @$keys), scalar(@$keys), @$keys};
    return [] if (!defined $retval);
    return $retval;
}

sub set {
    my ($self, $keys, $value) = @_;
    croak "Bad multi-key\n" unless ref $keys eq 'ARRAY';
    croak "set require an array reference\n" unless ref $value eq 'ARRAY';
    $self->{pack 'N' . ('w/a*' x @$keys), scalar(@$keys), @$keys} = [@$value];
}

sub rawput {
    my ($self, $keys, $value) = @_;
    croak "Bad multi-key\n" unless ref $keys eq 'ARRAY';
    $self->{pack 'N' . ('w/a*' x @$keys), scalar(@$keys), @$keys} = $value;
}

sub DELETE {
    my ($self, $keys) = @_;
    #croak "Bad multi-key\n" unless ref $keys eq 'ARRAY';
    $keys = [$keys eq '' ? ('') : split /$;/, $keys, -1] unless ref $keys eq 'ARRAY';#/
    @$keys or croak "Empty multi-key\n";
    delete $self->{pack 'N' . ('w/a*' x @$keys), scalar(@$keys), @$keys};
}

sub CLEAR {
    %{ shift() } = ();
}

sub EXISTS {
    my ($self, $keys) = @_;
    $keys = [$keys eq '' ? ('') : split /$;/, $keys, -1] unless ref $keys eq 'ARRAY';#/
    @$keys or croak "Empty multi-key\n";
    exists $self->{pack 'N' . ('w/a*' x @$keys), scalar(@$keys), @$keys};
}

sub FIRSTKEY {
    my ($self) = @_;
    keys %$self; # reset iterator
    $self->NEXTKEY;
}

sub NEXTKEY {
    my ($self) = @_;
    defined(my $key = each %$self) or return;
    my $n = unpack 'N', $key;
    [ unpack 'x4' . ('w/a*' x $n), $key ];
}

sub SCALAR {
    my ($self) = @_;
    scalar %$self;
}

*delete=*DELETE;

sub paths_prefix {
    my ($self, $pathptr) = @_;
    my $substr=pack 'w/a*' x @$pathptr, @$pathptr;
    my @path_at;
    while (my $key = each %$self) {
	if (index ($key, $substr, 4)==4) { # 4 is the sizeof(N)
	    my $n = unpack 'N', $key;
	    push @path_at, [ unpack 'x4' . ('w/a*' x $n), $key ];
	}
    }
    return @path_at;
}

sub paths_suffix {
    my ($self, $pathptr) = @_;
    my $substr=pack 'w/a*' x @$pathptr, @$pathptr;
    my @path_at;
    while (my $key = each %$self) {
	if (rindex ($key, $substr)==length($key)-length($substr)) { # at the end of string
	    my $n = unpack 'N', $key;
	    push @path_at, [ unpack 'x4' . ('w/a*' x $n), $key ];
	}
    }
    return @path_at;
}

sub paths_all {
    my ($self) = @_;
    my @paths;
    while (my $key = each %$self) {
	my $n = unpack 'N', $key;
	push @paths, [ unpack 'x4' . ('w/a*' x $n), $key ];
    }
    return @paths;
}

sub find {
    my ($self) = @_;
    Carp::croak "find is not implemented in ".ref($self);
}

1;

