#!/usr/bin/perl -w
# Created: Sat Mar 30 12:41:06 2002

package Data::Array2ArrayMap::Hash::Tree;
use Carp;
use strict;

use Data::Array2ArrayMap;
use Clone;
use Scalar::Util qw/reftype/;

use vars qw/$VERSION @ISA $class/;
$VERSION = '0.12';

# static include adaptation
use vars qw/ @ISA/;
BEGIN {
    $class='Data::Array2ArrayMap::Hash::Tree';
    @ISA = qw/Data::Array2ArrayMap Clone/;
}

sub is_readonly() {return 0}
sub can_store_empty_array() {return 1}
sub has_find() {return 1}
sub has_clone() {return 1}
sub has_keys_at() {return 1}
sub can_list_paths() {return 1}

sub new {
    my ($class, $HashOfHashes) = @_;
    my $selftype=ref $HashOfHashes;
    $HashOfHashes = {} if ! $HashOfHashes or ($selftype ne 'HASH' and $selftype ne $class);
    bless($HashOfHashes,$class);
    return $HashOfHashes;
}

sub get {
    my ($cur_node, $pathptr)=@_;
    my @path=@$pathptr;
    confess "get(@path): I will return ARRAY\n" unless (wantarray);
    # optimization for speed (length=1)
    my $key=shift @path;
    confess "bad path @path" unless $key;
    return unless (exists ($cur_node->{$key}));
    return @{$cur_node->{$key}} unless @path;
    $cur_node= $cur_node->{$key};
    # optimization for speed (length=2)
    $key=shift @path;
    return unless (exists ($cur_node->{$key}));
    return @{$cur_node->{$key}}  unless @path;
    $cur_node= $cur_node->{$key};
    # optimization for speed (length=3)
    $key=shift @path;
    return unless (exists ($cur_node->{$key}));
    return @{$cur_node->{$key}} unless @path;
    $cur_node= $cur_node->{$key};
    # no optimization for speed (length=oo)
    $cur_node = node($cur_node,@path);
    return unless ($cur_node);
    if (ref $cur_node eq 'ARRAY') {
	return @$cur_node;
    } else {
	confess "get is called with too long path";
    }	
}

sub set {
    my ($cur_node, $pathptr, $valueptr )= @_;
    croak "I want ARRAY of values\n" unless (ref $valueptr eq 'ARRAY');
    $cur_node=node_ch($cur_node,@$pathptr);
    if (ref $cur_node eq 'ARRAY') {
	@$cur_node=@$valueptr;
    } elsif (! defined $cur_node) {
	confess "function cache->set met internal error 2";
    } else {
	confess 
	    "function cache->set is called with wrong number of arguments";
    }	
}

# powerful version of exists
sub get_ref_if {
     my ($cur_node, $pathptr) = @_;
     my @path=@$pathptr;
     # optimization for speed (length=1)
     my $key=shift @path;
     confess "bad path @path" unless $key;
     return $cur_node->{$key} unless @path;
     $cur_node= $cur_node->{$key};
     # optimization for speed (length=2)
     $key=shift @path;
     return $cur_node->{$key} unless @path;
     $cur_node= $cur_node->{$key};
     # optimization for speed (length=3)
     $key=shift @path;
     return $cur_node->{$key} unless @path;
     $cur_node= $cur_node->{$key};
     # no optimization for speed (length=oo)
     ($cur_node) = node($cur_node,@path);
     unless($cur_node) {return;}
     if (ref $cur_node eq 'ARRAY') {
	 return $cur_node;
     } else {
	 confess "function get_ref_if is called with too short path";
     }
}

## argument path: deletes the whole branch after path
sub clear_prefix {
    my ($cache, $pathptr)=@_;
    if (@$pathptr==0 ) {
	%{$cache}=();
	return;
    }
    my @path=@$pathptr;
    my $key = pop @path;
    my ($cur_node) = node($cache,@path);
    return unless $cur_node;
    delete $cur_node->{$key};
}

*delete=*clear_prefix;

sub find {
    my ($cur_node, $pathptr)=@_;
    my @path=@$pathptr;
    confess "find: I will return ARRAY\n" unless (wantarray);
    my $value=pop @path;
    my $key;
    while ($key=shift @path) {
	return unless (exists ($cur_node->{$key}));
	confess "bad path!" if ref ($cur_node->{$key}) ne 'HASH';
	$cur_node= $cur_node->{$key};
    }
    my @found;		# Not in index...
    foreach my $obj (keys %{$cur_node}) {
	push @found, $obj if grep($_ eq $value,@{$cur_node->{$obj}});
    }
    return @found;
}

sub subpaths_at {
    my ($cache, $pathptr) = @_;
    my $cur_node = node ($cache, @$pathptr);
    return unless $cur_node;
    my @res;
    _rsubpath($cur_node,[], \@res);
    return @res;
}

sub paths_all {
    my ($self)=@_;
    return $self->subpaths_at([]);
}

# keys_at returns all nodes on argument level 
sub keys_at {
    my ($cache, $pathptr) = @_;
    return keys %$cache unless @$pathptr; # optimization
    my ($cur_node) = node ($cache, @$pathptr);
    return unless $cur_node;
    return keys %$cur_node;
}

########################## Iner subs ###############################

sub node_ch {
    my ($cur_node, @path)=@_;
    while (@path) {
	if ((reftype $cur_node eq 'HASH')) {
	    if (! defined $cur_node->{$path[0]}) {
		if (@path>1) {
		    $cur_node->{$path[0]}={};
		} else {
		    $cur_node->{$path[0]}=[];
		};
	    };
	    $cur_node=$cur_node->{shift @path};
	} elsif (reftype $cur_node eq 'ARRAY') {
	    confess "function node_ch is called with too long context";
	} else {
	    confess "function node_ch met internal error 1";
	}
    }
    return $cur_node;
}

sub node {
    my ($cur_node, @path)=@_;
    while (@path) {
	my $reftype = reftype $cur_node;
	if (($reftype eq 'HASH')) {
	    $cur_node=$cur_node->{shift @path};
	} elsif ($reftype eq 'ARRAY') {
	    return;
	} else {
	    return;
	}
    }
    return $cur_node;
}

sub _rsubpath {
    my ($cur_node,$path,$res) = @_;
    if (ref $cur_node eq "ARRAY") {
	my @path=@$path;
	push @$res, \@path;
    } else {
	foreach my $key (keys %$cur_node) {
	    push @$path, $key;
	    _rsubpath($cur_node->{$key}, $path, $res);
	    pop @$path;
	}
    }
}

# stub for tie
#sub SCALAR {
#    my ($self) = @_;
#    scalar %$self;
#}


1;
