package Data::Array2ArrayMap::Hash::XSTree;

use 5.005;
use strict;

require DynaLoader;
use vars qw($VERSION @ISA);

use Data::Array2ArrayMap;
use Clone;

$VERSION = '0.15';

@ISA = qw(Data::Array2ArrayMap DynaLoader Clone);

bootstrap Data::Array2ArrayMap::Hash::XSTree $VERSION;

*exists_at=*get_ref_if;

sub clone {return Clone::clone(shift())}

sub get_ref {
    my ($self, $pathptr)=@_;
    my $ret = $self->get_ref_if($pathptr);
    return $ret? $ret : [];
}

use Carp;
# static include adaptation
use vars qw/$class/;
BEGIN {
    $class='Data::Array2ArrayMap::Hash::XSTree';
}


sub is_readonly() {return 0}
sub can_store_empty_array() {return 1}
sub has_find() {return 1}
sub has_clone() {return 1}
sub has_keys_at() {return 1}
sub can_list_paths() {return 1}

sub new {
    my ($class, $HashOfHashes) = @_;
    my $selftype=ref $HashOfHashes;
    $HashOfHashes = {} if ! $HashOfHashes or ($selftype ne 'HASH' and $selftype ne $class);
    bless($HashOfHashes,$class);
    return $HashOfHashes;
}

## argument path: deletes the whole branch after path
sub clear_prefix {
    my ($cache, $pathptr)=@_;
    confess "array pointer is required" unless ref $pathptr;
    if (@$pathptr==0 ) {
	%{$cache}=();
	return;
    }
    my $cur_node = _node_at($cache,$pathptr);
    return unless $cur_node;
    delete $cur_node->{$pathptr->[$#{$pathptr}]};
}

*delete=*clear_prefix;

sub find {
    my ($cur_node, $pathptr)=@_;
    my @path=@$pathptr;
    confess "find: I will return ARRAY\n" unless (wantarray);
    $cur_node= $cur_node->_node_at($pathptr);
    return unless $cur_node;
    my $value=pop @path;
    my @found;		# Not in index...
    foreach my $obj (keys %{$cur_node}) {
	push @found, $obj if grep($_ eq $value,@{$cur_node->{$obj}});
    }
    return @found;
}

sub subpaths_at {
    my ($self, $pathptr) = @_;
    my $cur_node = $self->get_ref_if ($pathptr);
    return unless $cur_node;
    my @res;
    _rsubpath($cur_node,[], \@res);
    return @res;
}

sub paths_all {
    my ($self)=@_;
    return $self->subpaths_at([]);
}

# keys_at returns all nodes on argument level
sub keys_at {
    my ($cache, $pathptr) = @_;
    return keys %$cache unless @$pathptr; # optimization
    my $cur_node = get_ref_if ($cache, $pathptr);
    return unless $cur_node;
    return keys %$cur_node;
}

########################## Iner subs ###############################

sub _rsubpath {
    my ($cur_node,$path,$res) = @_;
    if (ref $cur_node eq "ARRAY") {
	my @path=@$path;
	push @$res, \@path;
    } else {
	foreach my $key (keys %$cur_node) {
	    push @$path, $key;
	    _rsubpath($cur_node->{$key}, $path, $res);
	    pop @$path;
	}
    }
}

# stub for tie
#sub SCALAR {
#    my ($self) = @_;
#    scalar %$self;
#}


1;
